package edu.chl.hajo.monopoly.core;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.List;

import edu.chl.hajo.monopoly.constants.Constants;
import edu.chl.hajo.monopoly.constants.Piece;
import edu.chl.hajo.monopoly.constants.StateChanges;
import edu.chl.hajo.monopoly.core.space.Property;
import edu.chl.hajo.monopoly.util.IObservable;

/**
 * A player in the game
 * 
 * @author hajo
 * 
 */
// Observed by PlayerPanel
// (previously by SpacePanel bad)
public class Player implements IObservable {

	private final PropertyChangeSupport observers = new PropertyChangeSupport(
			this);

	private final String name;
	private final Piece piece;
	private int position = 0;
	//TODO private int oldPosition = 0; Not needed?
	// private int ordering;
	// TODO
	//private boolean inJail = false;
	private int balance = Constants.PLAYER_BALANCE;
	private List<Property> properties = new ArrayList<Property>();
	
	public Player(String name, Piece piece) {
		super();
		this.name = name;
		this.piece = piece;
	}

	@Override
	public void addObserver(PropertyChangeListener observer) {
		observers.addPropertyChangeListener(observer);
	}

	// Relative move
	public void move(int steps) {
		position = (position + steps) % (Constants.MAX_SPACES);
		observers.firePropertyChange(
					StateChanges.PLAYER_POSITION.toString(), null, position);	
	}

	// Absolute move 
	public void moveTo(int position) {
		this.position = position;
		observers.firePropertyChange(
					StateChanges.PLAYER_POSITION.toString(), null, position);	
	}
	
	public int getBalance() {
		return balance;
	}

	public String getName(){ 
		return name;
	}

	// TODO Checks here balance < price?
	public void withDraw(int price) {
		balance -= price;
		observers.firePropertyChange(
				StateChanges.PLAYER_WITHDRAW.toString(), null, balance);
	}
	

	public void addIncome(int income) {
		balance += income;
		observers.firePropertyChange(
				StateChanges.PLAYER_ADD_INCOME.toString(), null, balance);
	}
	
	
	public int getPosition() {
		return position;
	}

	public Piece getPiece() {
		return piece;
	}
	
	public void addProperty(Property property) {
		properties.add(property);
	}

	public void removeProperty(Property property) {
		properties.remove(property);
	}

	public List<Property> getProperties() {
		return properties;
	}
}
