package edu.chl.hajo.monopoly.build;

import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;

import edu.chl.hajo.monopoly.constants.Constants;
import edu.chl.hajo.monopoly.core.Board2;
import edu.chl.hajo.monopoly.core.Dices;
import edu.chl.hajo.monopoly.core.Game2;
import edu.chl.hajo.monopoly.core.space.AbstractSpace;
import edu.chl.hajo.monopoly.core.space.NonProperty;
import edu.chl.hajo.monopoly.core.space.Property;
import edu.chl.hajo.monopoly.core.space.Street;
import edu.chl.hajo.monopoly.core.space.Tax;
import edu.chl.hajo.monopoly.io.FileReader;

/**
 * Building the model. Factored out from the model
 * i.e. the building is handled outside the model
 * 
 * @author hajo
 *
 */
public class GameBuilder {

	private List<String> spaceNames;
	private List<AbstractSpace> spaces;
	private Board2 board;
	private Dices dices;
	
	public GameBuilder(){
	}

	public Game2 build() throws FileNotFoundException {
		spaceNames = FileReader.getInstance().readDataFile(
				Constants.SPACES_DATA_FILE);
		spaces = buildSpaces(spaceNames);
		board = new Board2(spaces);
		dices = new Dices();
		Game2 game = new Game2(board, dices);
		game.setRules("DUMMY"); // TODO
		return game;
	}

	public List<String> getSpaceNames() {
		return spaceNames;
	}

	public List<AbstractSpace> getSpaces() {
		return spaces;
	}

	public Board2 getBoard() {
		return board;
	}

	public Dices getDices() {
		return dices;
	}

	private List<AbstractSpace> buildSpaces(List<String> spaceNames) {
		List<AbstractSpace> spaces = new ArrayList<AbstractSpace>();
		for (String s : spaceNames) {
			AbstractSpace as = null;
			if (s.matches("street.*")) {
				as = new Street(s);
			} else if (s.matches("(train|energy|water)")) {
				as = new Property(s);
			} else if (s.matches("(chance|go|jail|gojail|park|community)")) {
				as = new NonProperty(s);
			} else if (s.equals("tax")) {
				as = new Tax(s);
			}
			spaces.add(as);
		}
		return spaces;
	}

}
