package edu.chl.hajo.monopoly.view.details;

/**
 * Detailed view of Street (can buy/sell and buy/sell buildings)
 * @author hajo
 *
 */
 
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.WindowConstants;

import edu.chl.hajo.monopoly.constants.StateChanges;
import edu.chl.hajo.monopoly.core.space.Street;
import edu.chl.hajo.monopoly.ctrl.CF;
import edu.chl.hajo.monopoly.rules.Rules;

@SuppressWarnings("serial")
public class StreetDetails extends AbstractSpaceDetails implements
		PropertyChangeListener {

	private JButton jBtnSell;
	private JButton jBtnBuy;
	private JButton jBtnBuyBuilding;
	private JButton jBtnSellBuilding;

	private Street street;
	
	public StreetDetails(JPanel parent, Street street) {
		super(parent, street);
		this.street = street;
		
		eb.connect(this, street);
		initGUI();
		initControls();
	}
	
	@Override
	protected void onOpen() {
		setButtonsEnabled();
	}
		
	private void setButtonsEnabled(){
		jBtnBuy.setEnabled(Rules.buyEnabled(street));
		jBtnSell.setEnabled(Rules.sellStreetEnabled(street));
		jBtnBuyBuilding.setEnabled(Rules.buyBuildingEnabled(street));
		jBtnSellBuilding.setEnabled(Rules.sellBuildingEnabled(street));
	}
	
	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		String e = evt.getPropertyName();
		
		boolean b = e.equals(StateChanges.PROPERTY_GOT_OWNER.toString()) ||
				e.equals(StateChanges.PROPERTY_LOST_OWNER.toString()) || 
				e.equals(StateChanges.STREET_ADD_BUILDING.toString()) ||
				e.equals(StateChanges.STREET_REMOVE_BUILDING.toString()) ;
		if( b ){
			setButtonsEnabled();
		}
	}

	private void initControls() {
		jBtnBuy.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				CF.get(CF.Name.BUY_PROPERTY, street).execute();
			}
		});
		jBtnSell.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				CF.get(CF.Name.SELL_PROPERTY, street).execute();
			}
		});
		jBtnBuyBuilding.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				CF.get(CF.Name.BUY_BUILDING, street).execute();
			}
		});
		jBtnSellBuilding.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				CF.get(CF.Name.SELL_BUILDING, street).execute();
			}
		});
	}

	// -- GUI --------------------

	private void initGUI() {
		try {
			{
				FlowLayout thisLayout = new FlowLayout();
				getContentPane().setLayout(thisLayout);
				this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
				{
					jBtnBuy = new JButton();
					getContentPane().add(jBtnBuy);
					jBtnBuy.setText("JBtnBuy");
				}
				{
					jBtnSell = new JButton();
					getContentPane().add(jBtnSell);
					jBtnSell.setText("jBtnSell");
					jBtnSell.setEnabled(false);
				}

				{
					jBtnBuyBuilding = new JButton();
					getContentPane().add(jBtnBuyBuilding);
					jBtnBuyBuilding.setText("jBtnBuyBuilding");
					jBtnBuyBuilding.setEnabled(false);
				}
				{
					jBtnSellBuilding = new JButton();
					getContentPane().add(jBtnSellBuilding);
					jBtnSellBuilding.setText("jBtnSellBuilding");
					jBtnSellBuilding.setEnabled(false);
				}

			}
			{
				this.setSize(287, 261);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}
