package edu.chl.hajo.monopoly.ctrl;

import java.util.ArrayList;
import java.util.List;

import junit.framework.Assert;

import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import edu.chl.hajo.monopoly.build.GameBuilder;
import edu.chl.hajo.monopoly.constants.Piece;
import edu.chl.hajo.monopoly.core.IGame;
import edu.chl.hajo.monopoly.core.Locator;
import edu.chl.hajo.monopoly.core.Player;
import edu.chl.hajo.monopoly.core.space.AbstractSpace;
import edu.chl.hajo.monopoly.core.space.Street;
import edu.chl.hajo.monopoly.event.EventBus;
import edu.chl.hajo.monopoly.util.Config;

/**
 * Possible to do heavy testing of controls 
 * This is the main test (possible should split
 * over more test classes).
 * Not finished, many more test should be done.
 * @author hajo
 *
 */
public class TestCtrl {
	
	// Just so we don't need to use the Locator 
	// in the test methods
	static IGame g;
	
	// Must be run once (BeforeClass), can't call
	// Locator.set() multiple times
	@BeforeClass
	public static void setUpClass() throws Exception {
		g = new GameBuilder().build();
		// Must set this used by all controls
		Locator.set(g);
	}
	
	// Called before every method, fresh data
	// All player at GO (position 0)
	@Before
	public void setUp(){
		List<Player> ps = new ArrayList<Player>();
		ps.add(new Player("Olle", Piece.DOG));
		ps.add(new Player("Fia", Piece.TOP_HAT));
		ps.add(new Player("Pelle", Piece.CAR));
		ps.add(new Player("Sanna", Piece.IRON));
		// No events for now
		EventBus.getInstance().on(false);
		g.setPlayers(ps);
	}
	
	@Test
	public void testMoveAPlayer() {
		// Go to community
		new MoveCtrl(2).execute();
		
		Player p = g.getActualPlayer();
		AbstractSpace s = g.getActualSpace();
		
		Assert.assertTrue( s.isVisitingPlayer(p));
		Assert.assertTrue( s.getName().equals("community"));
		Assert.assertTrue( p.getName().equals("Olle"));		
	}
	
	@Test
	public void testEndTurn(){
		Player p = g.getActualPlayer();
		new EndTurnCtrl().execute();
		new EndTurnCtrl().execute();
		new EndTurnCtrl().execute();
		new EndTurnCtrl().execute();
		Assert.assertTrue( g.getActualPlayer() == p );		
	}
	
	@Test
	public void testMoveAndEndTurn() {
		new MoveCtrl().execute();
		new EndTurnCtrl().execute();
		
		Player p = g.getActualPlayer();
		Assert.assertTrue( p.getName().equals("Fia"));	
	}
	
	
	@Test
	public void testBuyProperty(){
		Player p = g.getActualPlayer();
		
		Assert.assertTrue(p.getName().equals("Olle"));
				
		// Know this is a street (i.e. a property)
		Street s = (Street) g.getSpaces().get(3);
		int oldBalance = p.getBalance();
		// Go to street
		new MoveCtrl(3).execute();
		new BuyPropertyCtrl(s).execute();
		
		Assert.assertTrue( s.getOwner() == p);
		Assert.assertTrue( p.getProperties().size() == 1);
		Assert.assertTrue( p.getBalance() == oldBalance - s.getPrice());		
	}

	@Test
	public void testMoveAndPassGo() {
		Player p = g.getActualPlayer();
		p.setBalance(0);
		// Must move 1 else (pos <= oldPos)
		// will fail (not possible to fail in real game)
		new MoveCtrl(1).execute();
		new MoveCtrl(40).execute();
		Assert.assertTrue( p.getBalance() >= Config.getPassGoIncome());	
	}
	
	@Test
	public void testForceWithDraw() {
		// This is also tested as an event
		Player olle = g.getActualPlayer();
		olle.setBalance(100);
		new ForceWithDrawCtrl(100).execute();
		Assert.assertTrue( olle.getBalance() == 0);
	}
	
}
