package edu.chl.hajo.monopoly.ctrl;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;

import org.junit.Assert;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import edu.chl.hajo.monopoly.build.GameBuilder;
import edu.chl.hajo.monopoly.constants.GameEvents;
import edu.chl.hajo.monopoly.constants.Piece;
import edu.chl.hajo.monopoly.core.IGame;
import edu.chl.hajo.monopoly.core.Locator;
import edu.chl.hajo.monopoly.core.Player;
import edu.chl.hajo.monopoly.core.space.AbstractSpace;
import edu.chl.hajo.monopoly.event.EventBus;

/**
 * Testing event based model 
 * @author hajo
 *
 */
public class TestEvents {

	// Just so we don't need to use the Locator
	// in the test methods
	static IGame g;

	// Must be run once (BeforeClass), can't call
	// Locator.set() multiple times
	@BeforeClass
	public static void setUpClass() throws Exception {
		g = new GameBuilder().build();
		Locator.set(g);
	}

	// Called before every method, fresh data
	@Before
	public void setUp() {
		List<Player> ps = new ArrayList<Player>();
		ps.add(new Player("Olle", Piece.DOG));
		ps.add(new Player("Fia", Piece.TOP_HAT));
		ps.add(new Player("Pelle", Piece.CAR));
		ps.add(new Player("Sanna", Piece.IRON));
		EventBus.getInstance().on(false);
		g.setPlayers(ps);
		EventBus.getInstance().on(true);
	}

	// Emulate observers
	private class Observer implements PropertyChangeListener {
		// Avoid exceptions
		{
			for (AbstractSpace as : g.getSpaces()) {
				EventBus.getInstance().connect(this, as);
			}
			for (Player p : g.getPlayers()) {
				EventBus.getInstance().connect(this, p);
			}
		}

		List<String> events = new ArrayList<String>();

		@Override
		public void propertyChange(PropertyChangeEvent evt) {
			events.add(evt.getPropertyName());
		}
	}

	@Test
	public void testStateChangesGame() {
		Observer o = new Observer();
		new EndTurnCtrl().execute();
		List<String> es = o.events;
		Assert.assertTrue(es.size() == 4);

	}

	@Test
	public void testForceWithDraw() {
		Observer o = new Observer();
		Player olle = g.getActualPlayer();
		olle.setBalance(0);
		new ForceWithDrawCtrl(100).execute();
		List<String> es = o.events;
		Assert.assertTrue(es.size() == 1);
		Assert.assertTrue(es.get(0).equals(
				GameEvents.PLAYER_MUST_SELL.toString()));
	}

}
