package edu.chl.hajo.monopoly.core;

import java.util.Collection;

import java.util.List;
import java.util.Random;

/**
 * Class representing the overall game
 *
 * @author hajo
 *
 */
public class Monopoly implements IMonopoly {

    private final List<Player> players;
    private final DicePair dicePair;
    private final Board board;
    private Player actual;

    public Monopoly(List<Player> players, Board board, DicePair dicePair) {
        this.dicePair = dicePair;
        this.board = board;
        this.players = players;
        // Always create a new game when starting
        actual = players.get(new Random().nextInt(players.size()));
    }

    @Override
    public void next() {

        int i = players.indexOf(actual);
        actual = players.get((i + 1) % players.size());
        dicePair.setEnabled(true);

    }

    @Override
    public void move() {

        Space oldPos = actual.getPosition();
        dicePair.roll();
        Space newPos = board.getSpace(oldPos, dicePair.getTotal());
        actual.setPosition(newPos);
        dicePair.setEnabled(dicePair.bothAre(6));

        if (board.hasPassedGo(oldPos, newPos)) {
            //System.out.println("passed go");
            actual.income(MonopolyOptions.getBonus());
        }

    }


// ------- Accessors -------------
@Override
        public Player getActual() {
        return actual;
    }

    @Override
        public Collection<Player> getPlayers() {
        return players;
    }

    @Override
        public DicePair getDicePair() {
        return dicePair;
    }

    @Override
        public Board getBoard() {
        return board;
    }

}
