package edu.chl.hajo.mply.core;

import java.util.ArrayList;
import java.util.List;


/**
 * Class representing the overall game (application layer, coordinating
 * all model objects to perform the actions intended by the user,
 * a singleton)
 * 
 * 
 * NOTE: This is very similar to the commandLine version split up
 * in different methods.
 * 
 * @author hajo
 * 
 */
public class Monopoly implements IMonopoly {
    //INSTANCE;
    
    private static IMonopoly instance;
    public static IMonopoly getInstance(){
        if( instance == null){
            instance = new Monopoly();
        }
        return instance;
    }

    private final Board board = new Board();
    private final List<Player> players = new ArrayList<Player>();
    private final Piece p1;
    private final Piece p2;
    private final CardBundle bundle = new CardBundle();
    private final Dice d = new Dice(P.INSTANCE.getMaxDice());
    private Piece actualPiece;

    private Monopoly() {
        players.add(new Player("pelle"));
        players.add(new Player("fia"));
        p1 = new Piece(players.get(0), Piece.Tag.CAR);
        p2 = new Piece(players.get(1), Piece.Tag.DOG);
        actualPiece = p1;
    }

    /* (non-Javadoc)
     * @see edu.chl.hajo.mply.core.IMonopoly#move()
     */
    @Override
    public void move() {
        if (!actualPiece.isMovable()) {
            return;
        }
        Player p = actualPiece.getPlayer();
        int i = p.roll(d);
        actualPiece.move(i);
        if (i < P.INSTANCE.getMaxDice()) {
            actualPiece.setMovable(false);
        }
        if (actualPiece.passedGo()) {
            actualPiece.getPlayer().income(P.INSTANCE.getPassGoIncome());
        }
        if (board.getSpace(actualPiece.getPosition()).isCard()) {
            p.pickCard(bundle);
        }
    }

    // TODO refactor
    private void move(int steps) {
        if (!actualPiece.isMovable()) {
            return;
        }
        Player p = actualPiece.getPlayer();
        // int i = p.roll(d);
        actualPiece.move(steps);
        actualPiece.setMovable(false);
        if (actualPiece.passedGo()) {
            actualPiece.getPlayer().income(P.INSTANCE.getPassGoIncome());
        }
        if (board.getSpace(actualPiece.getPosition()).isCard()) {
            p.pickCard(bundle);
        }
    }

    /* (non-Javadoc)
     * @see edu.chl.hajo.mply.core.IMonopoly#next()
     */
    @Override
    public void next() {
        actualPiece = (actualPiece == p1) ? p2 : p1;
        actualPiece.setMovable(true);
    }

    /* (non-Javadoc)
     * @see edu.chl.hajo.mply.core.IMonopoly#actOnCard(edu.chl.hajo.mply.core.Card)
     */
    @Override
    public void actOnCard(Card c) {
        if (c.getAction() == Card.Action.MOVE) {
            actualPiece.setMovable(true);
            move(c.getArg());
        } else if (c.getAction() == Card.Action.ROLL_AGAIN) {
            actualPiece.setMovable(true);
        } else {

        }
        actualPiece.getPlayer().actOnCard(c);
    }
}
