package edu.chl.hajo.monopoly03.model;

import java.util.Collection;
import java.util.List;

/**
 * Class representing the over all game
 * @author hajo
 * 
 */
public class Monopoly03 {

    private final List<Player> players;
    private final Dice dice;
    private final Board board;
    private final Deck deck;
    private Player actual;

    // First player will be actual
    public Monopoly03(Dice dice, Board board, List<Player> players, Deck deck) {
        this.dice = dice;
        this.board = board;
        this.players = players;
        this.deck = deck;
        actual = players.get(0);
    }

    public void move() {
        int steps = dice.roll();
        doMove(steps);
    }

    // Need this for fixed moves
    public void move(int steps) {
        doMove(steps);
    }

    private void doMove(int steps) {
        if (actual.isInJail() && steps < 12) {
            return;
        }
        Space old = board.remove(actual);
        Space s = board.get(old, steps);
        s.addVisitor(actual);

        Player owner = s.getOwner();
        if (owner != null && owner != actual) {
            actual.addDebt(new Debt(owner, s.getRent()));
        }
        /*
        if (s.isCardSpace()) {
            actual.addCard(deck.pickCard());
        }*/
    }

    public void endTurn() {
        int i = (players.indexOf(actual) + 1) % players.size();
        actual = players.get(i);
    }
    
    @Override
    public String toString(){
        // This seems simple to print??
        return dice.getFirst() + ":" + dice.getSecond() + board.toString();
    }

    public IBoard getBoard() {
        return board;
    }

    public Dice getDice() { // Possible interface 
        return dice;
    }
    
    public Player getActualPlayer() { // Possible interface 
        return actual;
    }
    
    public Collection<Player> getPlayers() { // Possible interface 
        return players;
    }
}
