package edu.chl.hajo.monopoly04.model;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;



/**
 * Class representing the over all game
 * @author hajo
 * 
 */
public class Monopoly04 implements IMonopoly {

    private final List<Player> players;
    private final Dice dice;
    private final Board board;
    private final Deck deck;
    private Player actual;

    public Monopoly04(Dice dice, Board board, List<Player> players, Deck deck) {
        this.dice = dice;
        this.board = board;
        this.players = players;
        this.deck = deck;
    }
    
    @Override
    public void start() {
        // TODO random
        actual = players.iterator().next();
        Space s = board.getSpaces().iterator().next();
        for( Player p : players){
            s.addVisitor(p);
        }
    }
    
    @Override
    public void move() {
        int steps = dice.roll();
        doMove(steps);
    }

    // Need this for fixed moves
    @Override
    public void move(int steps) {
        doMove(steps);
    }

    private void doMove(int steps) {
        // Possible move to Rules class
        if (actual.isInJail() && steps < 12) {
            return;
        }
        Space old = board.remove(actual);
        Space s = board.get(old, steps);
        s.addVisitor(actual);

        Player owner = s.getOwner();
        if (owner != null && owner != actual) {
            actual.addDebt(new Debt(owner, s.getRent()));
        }
    }

    @Override
    public void endTurn() {
        int i = (players.indexOf(actual) + 1) % players.size();
        actual = players.get(i);
    }
    
    @Override
    public String toString(){
        return dice.getFirst() + ":" + dice.getSecond() + board.toString();
    }

    public IBoard getBoard() {
        return board;
    }

    public Dice getDice() { 
        return dice;
    }
    
    public IPlayer getActualPlayer() { 
        return actual;
    }
    
    @SuppressWarnings("unchecked")
    public Collection<IPlayer> getPlayers() { 
        List<? extends IPlayer> pls = Collections.unmodifiableList(players);
        // Cast OK Player implements IPlayer
        return (Collection<IPlayer>) pls; 
    }
}
